@@ -1,7 +1,7 @@
 import copy
 import types
 
-from .util import six, Lexicon
+from .util import six, Lexicon, helpline
 
 from .config import merge_dicts, copy_dict
 from .parser import Context as ParserContext
@@ -10,7 +10,7 @@ from .tasks import Task
 
 class Collection(object):
     """
-    A collection of executable tasks.
+    A collection of executable tasks. See :doc:`/concepts/namespaces`.
     """
     def __init__(self, *args, **kwargs):
         """
@@ -119,13 +119,28 @@ class Collection(object):
         return method(obj, name=name)
 
     def __repr__(self):
+        task_names = list(self.tasks.keys())
+        collections = ["{}...".format(x) for x in self.collections.keys()]
         return "<Collection {!r}: {}>".format(
             self.name,
-            ", ".join(sorted(self.tasks.keys())),
+            ", ".join(sorted(task_names) + sorted(collections)),
         )
 
     def __eq__(self, other):
-        return self.name == other.name and self.tasks == other.tasks
+        return (
+            self.name == other.name and
+            self.tasks == other.tasks and
+            self.collections == other.collections
+        )
+
+    def __ne__(self, other):
+        return not self == other
+
+    def __nonzero__(self):
+        return self.__bool__()
+
+    def __bool__(self):
+        return bool(self.task_names)
 
     @classmethod
     def from_module(
@@ -152,6 +167,10 @@ class Collection(object):
         Explicitly given collections will only be given that module-derived
         name if they don't already have a valid ``.name`` attribute.
 
+        If the module has a docstring (``__doc__``) it is copied onto the
+        resulting `.Collection` (and used for display in help, list etc
+        output.)
+
         :param str name:
             A string, which if given will override any automatically derived
             collection name (or name set on the module's root namespace, if it
@@ -182,7 +201,9 @@ class Collection(object):
                 loaded_from=loaded_from,
                 auto_dash_names=auto_dash_names,
             )
-            return cls(*args, **kwargs)
+            instance = cls(*args, **kwargs)
+            instance.__doc__ = module.__doc__
+            return instance
         # See if the module provides a default NS to use in lieu of creating
         # our own collection.
         for candidate in ('ns', 'namespace'):
@@ -290,6 +311,16 @@ class Collection(object):
         rest = '.'.join(parts)
         return coll, rest
 
+    def subcollection_from_path(self, path):
+        """
+        Given a ``path`` to a subcollection, return that subcollection.
+        """
+        parts = path.split('.')
+        collection = self
+        while parts:
+            collection = collection.collections[parts.pop(0)]
+        return collection
+
     def __getitem__(self, name=None):
         """
         Returns task named ``name``. Honors aliases and subcollections.
@@ -421,10 +452,15 @@ class Collection(object):
     @property
     def task_names(self):
         """
-        Return all task identifiers for this collection as a dict.
+        Return all task identifiers for this collection as a one-level dict.
 
         Specifically, a dict with the primary/"real" task names as the key, and
         any aliases as a list value.
+
+        It basically collapses the namespace tree into a single
+        easily-scannable collection of invocation strings, and is thus suitable
+        for things like flat-style task listings or transformation into parser
+        contexts.
         """
         ret = {}
         # Our own tasks get no prefix, just go in as-is: {name: [aliases]}
@@ -441,7 +477,7 @@ class Collection(object):
                 ))
                 # Tack on collection name to alias list if this task is the
                 # collection's default.
-                if coll.default and coll.default == task_name:
+                if coll.default == task_name:
                     aliases += (coll_name,)
                 ret[self.subtask_name(coll_name, task_name)] = aliases
         return ret
@@ -478,3 +514,31 @@ class Collection(object):
         :returns: ``None``.
         """
         merge_dicts(self._configuration, options)
+
+    def serialized(self):
+        """
+        Return an appropriate-for-serialization version of this object.
+
+        See the documentation for `.Program` and its ``json`` task listing
+        format; this method is the driver for that functionality.
+        """
+        return {
+            "name": self.name,
+            "help": helpline(self),
+            "default": self.default,
+            "tasks": [
+                {
+                    "name": self.transform(x.name),
+                    "help": helpline(x),
+                    "aliases": [self.transform(y) for y in x.aliases],
+                }
+                for x in sorted(self.tasks.values(), key=lambda x: x.name)
+            ],
+            "collections": [
+                x.serialized()
+                for x in sorted(
+                    self.collections.values(),
+                    key=lambda x: x.name,
+                )
+            ],
+        }
