@@ -1,6 +1,7 @@
 from __future__ import unicode_literals, print_function
 
 import inspect
+import json
 import os
 import sys
 import textwrap
@@ -13,8 +14,8 @@ from .parser import Parser, ParserContext, Argument
 from .exceptions import (
     UnexpectedExit, CollectionNotFound, ParseError, Exit,
 )
-from .util import debug, enable_logging, sort_names
-from .platform import pty_size
+from .terminals import pty_size
+from .util import debug, enable_logging, helpline
 
 
 class Program(object):
@@ -73,9 +74,19 @@ class Program(object):
             ),
             Argument(
                 names=('list', 'l'),
-                kind=bool,
-                default=False,
-                help="List available tasks."
+                optional=True,
+                help="List available tasks, optionally limited to a namespace."
+            ),
+            Argument(
+                names=('list-depth', 'D'),
+                kind=int,
+                default=0,
+                help="When listing tasks, only show the first INT levels.",
+            ),
+            Argument(
+                names=('list-format', 'F'),
+                help="Change the display format used when listing tasks. Should be one of: flat (default), nested, json.", # noqa
+                default='flat',
             ),
             Argument(
                 names=('pty', 'p'),
@@ -119,14 +130,16 @@ class Program(object):
                 help="Disable task deduplication."
             ),
             Argument(
-                names=('root', 'r'),
+                names=('search-root', 'r'),
                 help="Change root directory used for finding task modules."
             ),
         ]
 
     # Other class-level global variables a subclass might override sometime
     # maybe?
-    indent_width = 2
+    leading_indent_width = 2
+    leading_indent = " " * leading_indent_width
+    indent_width = 4
     indent = " " * indent_width
     col_padding = 3
 
@@ -258,13 +271,13 @@ class Program(object):
             strings, or a string. See `.normalize_argv` for details.
 
         :param bool exit:
-            When ``True`` (default: ``False``), will ignore `.ParseError`,
+            When ``False`` (default: ``True``), will ignore `.ParseError`,
             `.Exit` and `.Failure` exceptions, which otherwise trigger calls to
             `sys.exit`.
 
             .. note::
                 This is mostly a concession to testing. If you're setting this
-                to ``True`` in a production setting, you should probably be
+                to ``False`` in a production setting, you should probably be
                 using `.Executor` and friends directly instead!
         """
         try:
@@ -298,6 +311,8 @@ class Program(object):
             # problems.
             if isinstance(e, ParseError):
                 print(e, file=sys.stderr)
+            if isinstance(e, Exit) and e.message:
+                print(e.message, file=sys.stderr)
             if isinstance(e, UnexpectedExit) and e.result.hide:
                 print(e, file=sys.stderr, end='')
             # Terminate execution unless we were told not to.
@@ -354,6 +369,13 @@ class Program(object):
                 self.print_help()
                 raise Exit
             self.load_collection()
+        # Set these up for potential use later when listing tasks
+        # TODO: be nice if these came from the config...! Users would love to
+        # say they default to nested for example. Easy 2.x feature-add.
+        self.list_root = None
+        self.list_depth = None
+        self.list_format = 'flat'
+        self.scoped_collection = self.collection
 
         # TODO: load project conf, if possible, gracefully
 
@@ -382,7 +404,19 @@ class Program(object):
                 raise ParseError("No idea what '{}' is!".format(halp))
 
         # Print discovered tasks if necessary
-        if self.args.list.value:
+        list_root = self.args.list.value # will be True or string
+        self.list_format = self.args['list-format'].value
+        self.list_depth = self.args['list-depth'].value
+        if list_root:
+            # Not just --list, but --list some-root - do moar work
+            if isinstance(list_root, six.string_types):
+                self.list_root = list_root
+                try:
+                    sub = self.collection.subcollection_from_path(list_root)
+                    self.scoped_collection = sub
+                except KeyError:
+                    msg = "Sub-collection '{}' not found!"
+                    raise Exit(msg.format(list_root))
             self.list_tasks()
             raise Exit
 
@@ -501,7 +535,7 @@ class Program(object):
         """
         # NOTE: start, coll_name both fall back to configuration values within
         # Loader (which may, however, get them from our config.)
-        start = self.args.root.value
+        start = self.args['search-root'].value
         loader = self.loader_class(config=self.config, start=start)
         coll_name = self.args.collection.value
         try:
@@ -518,11 +552,7 @@ class Program(object):
                 auto_dash_names=self.config.tasks.auto_dash_names,
             )
         except CollectionNotFound as e:
-            six.print_(
-                "Can't find any collection named {!r}!".format(e.name),
-                file=sys.stderr
-            )
-            raise Exit(1)
+            raise Exit("Can't find any collection named {!r}!".format(e.name))
 
     def parse_tasks(self):
         """
@@ -560,48 +590,150 @@ class Program(object):
             # Really wish textwrap worked better for this.
             for line in docstring.splitlines():
                 if line.strip():
-                    print(self.indent + line)
+                    print(self.leading_indent + line)
                 else:
                     print("")
             print("")
         else:
-            print(self.indent + "none")
+            print(self.leading_indent + "none")
             print("")
         print("Options:")
         if tuples:
             self.print_columns(tuples)
         else:
-            print(self.indent + "none")
+            print(self.leading_indent + "none")
             print("")
 
     def list_tasks(self):
-        # Sort in depth, then alpha, order
-        task_names = self.collection.task_names
-        # Short circuit if no tasks to show
-        if not task_names:
+        # Short circuit if no tasks to show (Collection now implements bool)
+        focus = self.scoped_collection
+        if not focus:
             msg = "No tasks found in collection '{}'!"
-            print(msg.format(self.collection.name))
-            raise Exit
-        pairs = []
-        for primary in sort_names(task_names):
-            # Add aliases
-            aliases = sort_names(task_names[primary])
-            name = primary
-            if aliases:
-                name += " ({})".format(', '.join(aliases))
-            # Add docstring 1st lines
-            task = self.collection[primary]
-            help_ = ""
-            if task.__doc__:
-                help_ = task.__doc__.lstrip().splitlines()[0]
-            pairs.append((name, help_))
+            raise Exit(msg.format(focus.name))
+        # TODO: now that flat/nested are almost 100% unified, maybe rethink
+        # this a bit?
+        getattr(self, "list_{}".format(self.list_format))()
 
-        # Print
+    def list_flat(self):
+        pairs = self._make_pairs(self.scoped_collection, ancestors=[])
+        self.display_with_columns(pairs=pairs)
+
+    def list_nested(self):
+        pairs = self._make_pairs(self.scoped_collection, ancestors=[])
+        extra = "'*' denotes collection defaults"
+        self.display_with_columns(pairs=pairs, extra=extra)
+
+    def _make_pairs(self, coll, ancestors):
+        pairs = []
+        indent = len(ancestors) * self.indent
+        # NOTE: skip 1st ancestor as it's always the root & thus
+        # implicit/unnamed; and also remember to loop in current coll
+        # (again unless it's itself the root)
+        display_ancestors = ancestors[1:]
+        if ancestors:
+            display_ancestors.append(coll)
+        ancestor_path = '.'.join(x.name for x in display_ancestors)
+        for name, task in sorted(six.iteritems(coll.tasks)):
+            is_default = name == coll.default
+            # Start with just the name and just the aliases, no prefixes or
+            # dots.
+            displayname = name
+            aliases = list(map(coll.transform, sorted(task.aliases)))
+            # If displaying a sub-collection (or if we are displaying a given
+            # namespace/root), tack on some dots to make it clear these names
+            # require dotted paths to invoke.
+            if ancestors or self.list_root:
+                displayname = ".{}".format(displayname)
+                aliases = [".{}".format(x) for x in aliases]
+            # Nested? Indent, and add asterisks to default-tasks.
+            if self.list_format == 'nested':
+                prefix = indent
+                if is_default:
+                    displayname += "*"
+            # Flat? Prefix names and aliases with ancestor names to get full
+            # dotted path; and give default-tasks their collection name as the
+            # first alias.
+            if self.list_format == 'flat':
+                prefix = ancestor_path
+                # Make sure leading dots are present for subcollections if
+                # scoped display
+                if prefix and self.list_root:
+                    prefix = '.' + prefix
+                aliases = [prefix + alias for alias in aliases]
+                if is_default and ancestors:
+                    aliases.insert(0, prefix)
+            # Generate full name and help columns and add to pairs.
+            alias_str = " ({})".format(", ".join(aliases)) if aliases else ""
+            full = prefix + displayname + alias_str
+            pairs.append((full, helpline(task)))
+        # Determine whether we're at max-depth or not
+        truncate = self.list_depth and (len(ancestors) + 1) >= self.list_depth
+        for name, subcoll in sorted(six.iteritems(coll.collections)):
+            displayname = name
+            if ancestors or self.list_root:
+                displayname = ".{}".format(displayname)
+            if truncate:
+                tallies = [
+                    "{} {}".format(len(getattr(subcoll, attr)), attr)
+                    for attr in ('tasks', 'collections')
+                    if getattr(subcoll, attr)
+                ]
+                displayname += " [{}]".format(", ".join(tallies))
+            if self.list_format == 'nested':
+                pairs.append((indent + displayname, helpline(subcoll)))
+            elif self.list_format == 'flat' and truncate:
+                # NOTE: only adding coll-oriented pair if limiting by depth
+                pairs.append((ancestor_path + displayname, helpline(subcoll)))
+            # Recurse, if not already at max depth
+            if not truncate:
+                recursed_pairs = self._make_pairs(subcoll, ancestors + [coll])
+                pairs.extend(recursed_pairs)
+        return pairs
+
+    def list_json(self):
+        # Sanity: we can't cleanly honor the --list-depth argument without
+        # changing the data schema or otherwise acting strangely; and it also
+        # doesn't make a ton of sense to limit depth when the output is for a
+        # script to handle. So we just refuse, for now. TODO: find better way
+        if self.list_depth:
+            raise Exit("The --list-depth option is not supported with JSON format!") # noqa
+        # TODO: consider using something more formal re: the format this emits,
+        # eg json-schema or whatever. Would simplify the
+        # relatively-concise-but-only-human docs that currently describe this.
+        coll = self.scoped_collection
+        data = coll.serialized()
+        print(json.dumps(data))
+
+    def task_list_opener(self, extra=""):
+        root = self.list_root
+        depth = self.list_depth
+        specifier = " '{}'".format(root) if root else ""
+        tail = ""
+        if depth or extra:
+            depthstr = "depth={}".format(depth) if depth else ""
+            joiner = "; " if (depth and extra) else ""
+            tail = " ({}{}{})".format(depthstr, joiner, extra)
+        text = "Available{} tasks{}".format(specifier, tail)
+        # TODO: do use cases w/ bundled namespace want to display things like
+        # root and depth too? Leaving off for now...
         if self.namespace is not None:
-            print("Subcommands:\n")
-        else:
-            print("Available tasks:\n")
+            text = "Subcommands"
+        return text
+
+    def display_with_columns(self, pairs, extra=""):
+        root = self.list_root
+        print("{}:\n".format(self.task_list_opener(extra=extra)))
         self.print_columns(pairs)
+        # TODO: worth stripping this out for nested? since it's signified with
+        # asterisk there? ugggh
+        default = self.scoped_collection.default
+        if default:
+            specific = ""
+            if root:
+                specific = " '{}'".format(root)
+                default = ".{}".format(default)
+            # TODO: trim/prefix dots
+            print("Default{} task: {}\n".format(specific, default))
 
     def print_columns(self, tuples):
         """
@@ -615,18 +747,20 @@ class Program(object):
         desc_width = (
             pty_size()[0]
             - name_width
-            - self.indent_width
+            - self.leading_indent_width
             - self.col_padding
             - 1
         )
         wrapper = textwrap.TextWrapper(width=desc_width)
         for name, help_str in tuples:
+            if help_str is None:
+                help_str = ""
             # Wrap descriptions/help text
             help_chunks = wrapper.wrap(help_str)
             # Print flag spec + padding
             name_padding = name_width - len(name)
             spec = ''.join((
-                self.indent,
+                self.leading_indent,
                 name,
                 name_padding * ' ',
                 self.col_padding * ' '
